@@ -24,9 +24,11 @@
 #include <config.h>
 #include <atk/atkcomponent.h>
 #include <atk/atktext.h>
-#include <glib/gi18n.h>
+#include <atk/atkhypertext.h>
+#include <atk/atkhyperlink.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gmacros.h>
-
+#include <pango/pango.h>
 #include "gtkhtml.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
@@ -41,6 +43,7 @@
 #include "object.h"
 #include "html.h"
 #include "text.h"
+#include "hyperlink.h"
 
 static void html_a11y_text_class_init    (HTMLA11YTextClass *klass);
 static void html_a11y_text_init          (HTMLA11YText *a11y_text);
@@ -74,6 +77,14 @@ static void html_a11y_text_get_character_extents (AtkText *text, gint offset,
 static gint html_a11y_text_get_offset_at_point (AtkText *text, gint x, gint y,
 					AtkCoordType coords);
 
+/* Hyperlink interface */
+static void atk_hyper_text_interface_init (AtkHypertextIface *iface);
+
+static AtkHyperlink * html_a11y_text_get_link (AtkHypertext *hypertext, gint link_index);
+static gint html_a11y_text_get_n_links (AtkHypertext *hypertext);
+static gint html_a11y_text_get_link_index (AtkHypertext *hypertext, gint char_index);
+
+
 /* Editable text interface. */
 static void 	atk_editable_text_interface_init      (AtkEditableTextIface *iface);
 static void	html_a11y_text_set_text_contents	(AtkEditableText      *text,
@@ -96,6 +107,11 @@ static void	html_a11y_text_paste_text	(AtkEditableText      *text,
 
 static AtkStateSet* html_a11y_text_ref_state_set	(AtkObject	*accessible);
 
+static AtkAttributeSet* html_a11y_text_get_run_attributes	(AtkText *text, 
+								 gint offset, 
+								 gint *start_offset, 
+								 gint *end_offset);
+
 static AtkObjectClass *parent_class = NULL;
 
 static gint
@@ -179,17 +195,24 @@ html_a11y_text_get_type (void)
 			NULL
 		};
 
+		static const GInterfaceInfo atk_hyper_text_info = {
+			(GInterfaceInitFunc) atk_hyper_text_interface_init,
+			(GInterfaceFinalizeFunc) NULL,
+			NULL
+		};
 
 		type = g_type_register_static (G_TYPE_HTML_A11Y, "HTMLA11YText", &tinfo, 0);
 		g_type_add_interface_static (type, ATK_TYPE_COMPONENT, &atk_component_info);
 		g_type_add_interface_static (type, ATK_TYPE_TEXT, &atk_text_info);
 		g_type_add_interface_static (type, ATK_TYPE_EDITABLE_TEXT, &atk_editable_text_info);
 		g_type_add_interface_static (type, ATK_TYPE_ACTION, &atk_action_info);
+		g_type_add_interface_static (type, ATK_TYPE_HYPERTEXT, &atk_hyper_text_info);
 	}
 
 	return type;
 }
 
+
 static void 
 atk_component_interface_init (AtkComponentIface *iface)
 {
@@ -220,6 +243,7 @@ atk_text_interface_init (AtkTextIface *iface)
 	iface->set_caret_offset = html_a11y_text_set_caret_offset;
 	iface->get_character_extents = html_a11y_text_get_character_extents;
 	iface->get_offset_at_point = html_a11y_text_get_offset_at_point;
+	iface->get_run_attributes = html_a11y_text_get_run_attributes;
 }
 
 static void
@@ -514,29 +538,39 @@ html_a11y_text_get_text_at_offset (AtkText *text, gint offset, AtkTextBoundary b
 	switch (boundary_type) {
 	case ATK_TEXT_BOUNDARY_LINE_START:
 		start_slave = html_text_get_slave_at_offset (to, NULL, offset);
-		g_return_val_if_fail (start_slave, NULL);
-		end_slave = (HTMLTextSlave *) HTML_OBJECT (start_slave)->next;
 
-		if (end_slave && HTML_IS_TEXT_SLAVE (end_slave)) {
-			*end_offset = end_slave->posStart;
+		if (start_slave == NULL) {
+			*start_offset = 0;
+			*end_offset = to->text_len;
 		} else {
-			*end_offset = start_slave->posStart + start_slave->posLen;
+			end_slave = (HTMLTextSlave *) HTML_OBJECT (start_slave)->next;
+
+			if (end_slave && HTML_IS_TEXT_SLAVE (end_slave)) {
+				*end_offset = end_slave->posStart;
+			} else {
+				*end_offset = start_slave->posStart + start_slave->posLen;
+			}
+			*start_offset = start_slave->posStart;
 		}
-		*start_offset = start_slave->posStart;
 
 		return html_a11y_text_get_text (text, *start_offset, *end_offset);
 
 	case ATK_TEXT_BOUNDARY_LINE_END:
 		end_slave = html_text_get_slave_at_offset (to, NULL, offset);
-		g_return_val_if_fail (end_slave, NULL);
-		start_slave = (HTMLTextSlave *) HTML_OBJECT (end_slave)->prev;
 
-		if (start_slave && HTML_IS_TEXT_SLAVE (start_slave)) {
-			*start_offset = start_slave->posStart + start_slave->posLen;
+		if (end_slave == NULL) {
+			*start_offset = 0;
+			*end_offset = to->text_len;
 		} else {
-			*start_offset = end_slave->posStart;
+			start_slave = (HTMLTextSlave *) HTML_OBJECT (end_slave)->prev;
+
+			if (start_slave && HTML_IS_TEXT_SLAVE (start_slave)) {
+				*start_offset = start_slave->posStart + start_slave->posLen;
+			} else {
+				*start_offset = end_slave->posStart;
+			}
+			*end_offset = end_slave->posStart + end_slave->posLen;
 		}
-		*end_offset = end_slave->posStart + end_slave->posLen;
 
 		return html_a11y_text_get_text (text, *start_offset, *end_offset);
 
@@ -682,24 +716,171 @@ html_a11y_text_set_selection (AtkText *text, gint selection_num, gint start_offs
 }
 
 
-/*
-  AtkAttributeSet* (* get_run_attributes)         (AtkText	    *text,
-						   gint	  	    offset,
-						   gint             *start_offset,
-						   gint	 	    *end_offset);
-  AtkAttributeSet* (* get_default_attributes)     (AtkText	    *text);
-  void           (* get_character_extents)        (AtkText          *text,
-                                                   gint             offset,
-                                                   gint             *x,
-                                                   gint             *y,
-                                                   gint             *width,
-                                                   gint             *height,
-                                                   AtkCoordType	    coords);
-  gint           (* get_offset_at_point)          (AtkText          *text,
-                                                   gint             x,
-                                                   gint             y,
-                                                   AtkCoordType	    coords);
+/* Most of the code of the following function is copied from */
+/* libgail-util/gailmisc.c gail_misc_layout_get_run_attributes */
 
+static AtkAttributeSet *
+html_a11y_text_get_run_attributes (AtkText *text, 
+				   gint offset, 
+				   gint *start_offset, 
+				   gint *end_offset)
+{
+	PangoAttrIterator *iter;
+	PangoAttrList *attr;  
+	PangoAttrString *pango_string;
+	PangoAttrInt *pango_int;
+	PangoAttrColor *pango_color;
+	PangoAttrLanguage *pango_lang;
+	PangoAttrFloat *pango_float;
+	gint index, start_index, end_index;
+	gboolean is_next = TRUE;
+	gchar *value = NULL;
+	glong len;
+	gchar *textstring;
+	AtkAttributeSet *attrib_set = NULL;
+	HTMLText *t = HTML_TEXT (HTML_A11Y_HTML (text));	
+	
+	g_return_val_if_fail (t, NULL);
+
+	textstring = t->text;
+	attr = t->attr_list;
+
+	g_return_val_if_fail (attr && textstring, NULL);
+
+	len = g_utf8_strlen (textstring, -1);
+	iter = pango_attr_list_get_iterator (attr);
+
+	/* Get invariant range offsets */
+	/* If offset out of range, set offset in range */
+	if (offset > len)
+		offset = len;
+	else if (offset < 0)
+		offset = 0;
+
+	index = g_utf8_offset_to_pointer (textstring, offset) - textstring;
+	pango_attr_iterator_range (iter, &start_index, &end_index);
+
+	while (is_next) {
+		if (index >= start_index && index < end_index) {
+			*start_offset = g_utf8_pointer_to_offset (textstring, textstring + start_index);  
+			if (end_index == G_MAXINT)
+			/* Last iterator */
+			end_index = len;
+
+			*end_offset = g_utf8_pointer_to_offset (textstring, textstring + end_index);  
+			break;
+		}  
+		is_next = pango_attr_iterator_next (iter);
+		pango_attr_iterator_range (iter, &start_index, &end_index);
+	}
+
+	/* Get attributes */
+	if ((pango_string = (PangoAttrString*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_FAMILY)) != NULL) {
+		value = g_strdup_printf("%s", pango_string->value);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_FAMILY_NAME, 
+				value);
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_STYLE)) != NULL) {
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_STYLE, 
+				g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_STYLE, 
+									pango_int->value)));
+	} 
+
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_WEIGHT)) != NULL) {
+		value = g_strdup_printf("%i", pango_int->value);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_WEIGHT, 
+				value);
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_VARIANT)) != NULL) {
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_VARIANT, 
+				g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_VARIANT, 
+									pango_int->value)));
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_STRETCH)) != NULL) {
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_STRETCH, 
+				g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_STRETCH, 
+									pango_int->value)));
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_SIZE)) != NULL) {
+		value = g_strdup_printf("%i", pango_int->value / PANGO_SCALE);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_SIZE,
+				value);
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_UNDERLINE)) != NULL) {
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_UNDERLINE, 
+				g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_UNDERLINE, 
+									pango_int->value)));
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_STRIKETHROUGH)) != NULL) {
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_STRIKETHROUGH, 
+				g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_STRIKETHROUGH, 
+									pango_int->value)));
+	} 
+	if ((pango_int = (PangoAttrInt*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_RISE)) != NULL) {
+		value = g_strdup_printf("%i", pango_int->value);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_RISE,
+				value);
+	} 
+	if ((pango_lang = (PangoAttrLanguage*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_LANGUAGE)) != NULL) {
+		value = g_strdup( pango_language_to_string( pango_lang->value));
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_LANGUAGE, 
+				value);
+	} 
+	if ((pango_float = (PangoAttrFloat*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_SCALE)) != NULL) {
+		value = g_strdup_printf("%g", pango_float->value);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_SCALE, 
+				value);
+	} 
+	if ((pango_color = (PangoAttrColor*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_FOREGROUND)) != NULL) {
+		value = g_strdup_printf ("%u,%u,%u", 
+				pango_color->color.red, 
+				pango_color->color.green, 
+				pango_color->color.blue);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_FG_COLOR, 
+				value);
+	} 
+	if ((pango_color = (PangoAttrColor*) pango_attr_iterator_get (iter, 
+						PANGO_ATTR_BACKGROUND)) != NULL) {
+		value = g_strdup_printf ("%u,%u,%u", 
+				pango_color->color.red, 
+				pango_color->color.green, 
+				pango_color->color.blue);
+		attrib_set = gail_misc_add_attribute (attrib_set, 
+				ATK_TEXT_ATTR_BG_COLOR, 
+				value);
+	} 
+
+	pango_attr_iterator_destroy (iter);
+	return attrib_set;
+
+}
+
+/*
+  AtkAttributeSet* (* get_default_attributes)     (AtkText	    *text);
 */
 
 static gint
@@ -930,3 +1111,40 @@ html_a11y_text_paste_text	(AtkEditableText *text,
 
         g_signal_emit_by_name(html, "grab_focus");
 }
+
+
+static void
+atk_hyper_text_interface_init (AtkHypertextIface *iface)
+{
+	g_return_if_fail (iface != NULL);
+
+	iface->get_link = html_a11y_text_get_link;
+	iface->get_n_links = html_a11y_text_get_n_links;
+	iface->get_link_index = html_a11y_text_get_link_index;
+}
+
+/*
+ * AtkHyperLink interface
+ */
+
+static AtkHyperlink *
+html_a11y_text_get_link (AtkHypertext *hypertext, gint link_index)
+{
+	return html_a11y_hyper_link_new (HTML_A11Y (hypertext), link_index);
+}
+
+static gint
+html_a11y_text_get_n_links (AtkHypertext *hypertext)
+{
+	HTMLText *text = HTML_TEXT (HTML_A11Y_HTML (hypertext));
+	return g_slist_length (text->links);
+}
+
+static gint
+html_a11y_text_get_link_index (AtkHypertext *hypertext, gint char_index)
+{
+	HTMLObject *obj = HTML_A11Y_HTML (hypertext);
+	Link *link = html_text_get_link_at_offset (HTML_TEXT (obj), char_index);
+
+	return link ? link->start_offset : -1;
+}
