@@ -27,7 +27,11 @@
 #include <config.h>
 #endif
 
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+#include <glib/gi18n-lib.h>
+#else
 #include <glib/gi18n.h>
+#endif
 #include <string.h>
 #include <gnome.h>
 #include <bonobo.h>
@@ -70,7 +74,7 @@
 #include "text.h"
 #include "paragraph.h"
 #include "body.h"
-#include "spell.h"
+#include "spellchecker.h"
 #include "html-stream-mem.h"
 
 #include "gtkhtmldebug.h"
